function [MV,b,hbtype,Lambda]=matrixhbo(namematrix)
%MV=matrixhbo(namematrix) 
%  Defines the function that performes the matrix-vector multiplication
%  with a matrix (called A here). A matrix in Harwell-Boeing format
%  with the name namematrix (namematrix is a string) is converted
%  to a matrix A in MATLAB format (that is, to an n x n array of numerics).
%  MV is the handle of this function, n is the dimension of the matrix A.
%
%[MV,b]=matrixhbo(namematrix) 
%  Defines the right-hand side vector b of the linear problem A*x=b.
%
%Note that size(b,1) reveals the dimension of the problem
%
%Note: 
%  matrices in the "Matrix Market" can be downloaded in Harwell-Boeing format. 
%
%[MV,b,hbtype]=matrixhbo(namematrix)
%  b is the right-hand side vector as given in the Harwell-Boeing collection,
%  hbtype characterizes the matrix type.
%  
%[MV,b,hbtype,Lambda]=matrixhbo(namematrix)
%   Lambda is an n x 1 array of eigenvalues of A
%  (returns Lambda=[] if n is too large to allow efficient computation).
% 
%hbtype: matrix type, which will be one of the following three-letter
%	      codes: RSA, RZA, RUA, CSA, RRA, PSA, PSE, PUA, PRA, UNK
%   RSA  - Real symmetric
%   RZA  - Real skew symmetric
%   RUA  - Real unsymmetric
%   CSA  - Complex symmetric
%   RRA  - Real rectangular
%   PSA  - Pattern symmetric
%   PSE  - Pattern symmetric unassembled
%   PUA  - Pattern unsymmetric
%   PRA  - Pattern rectangular
%
% See also ReadMe

   if nargin==0, namematrix='meier01'; end    
   [A,xy,b,hbtype,rhstype]=hbo(namematrix);
   [n,m]=size(A);
  
   MV=@MyMV;
  
   %%% Compute eigenvalues, if required and n is not too large
   if nargout>3
     Lambda=[];
     if n<2001, Lambda=eig(full(A)); end 
   end

   function b=MyMV(x)
      b=A*x; 
   end

end