function [MV,b,typeMV]=problem0(n,flag)
%[MV,b]=problem0(n) 
%   Defines the right-hand side vector b of the linear problem A*x=b, and
%   defines the function that performes the matrix-vector multiplication
%   MV is the handle of this function, n is a parameter that determines 
%   the dimension of the matrix A.
%  
%Note that size(b,1) will reveal the dimension of the problem.
%
%Note: a function-handle is used to avoid repeatedly defining the matrix A.
%   See MATLAB's documentation "Using Handles to Store Function Parameters".
%
  
  if nargin<2, flag=1; end
  
  %%% Define the matrix H
  %%% Run "help gallery" or for information on matrices in the set 
  %%% "gallery" of test matrices available in Matlab,
  %%% run "help private/poisson" for information on the specific matrix
  %%% called poisson (from a discretization of Poisson's Equation).
  
  H=gallery('poisson',n);
  
  %%% determine the dimension of the problem: H is a sparse N x N matrix.
  
  N=size(H,1);
  
  %%% Determine b.
  %%% In practise b is given. However, for test purposes, it is convenient
  %%% to define b from a convenient solution x (which, of course, generally
  %%% is the quatity that has to be computed)
  
  x=ones(N,1); b=H*x;
  
  function c=MyMV(u)
     c=H*u;
  end
  
  MV=@MyMV; 
  typeMV=sprintf('Poisson matrix, poisson(%d)',n);
  
  if flag==0, return, end
  
  %%% Here we change the problem to demonstate the advantage of using 
  %%% functions to perform the matrix-vector multiplication rather
  %%% than using matrices: here, we "precondition" H by it's upper and lower
  %%% triangular parts (including diagonal), leading to the matrix A.
   
  L=tril(H);
  U=triu(H);
  D=diag(diag(H));
  
  %%% If y solves D*L\(H/U)y=D*(L\b), then x:=U\y solves H*x=b.
  b=D*(L\b);

  %%% Define the multiplication by A:=D*(L\(H/U)).
  function c=MyPMV(u)
     c=D*(L\(H*(U\u)));
  end
  
  MV=@MyPMV; 
  typeMV=sprintf('preconditioned MV');

  if flag==1, return, end
  
  tic
  A=D*(L\(H/U));
  toc
    
  %%% Define the matrix-vector multiplication by explicit use of A
  function c=MyMatrixPMV(u)
      c=A*u;
  end
  
  %%% return the handle for the preconditioned MV:
  MV=@MyMatrixPMV;
  typeMV=sprintf('preconditioned matrix',n);


end